@@ -121,7 +121,6 @@ class CHPControlMultiEnergy(G2PControlMultiEnergy):
             in_service=in_service, order=order, level=level, drop_same_existing_ctrl=drop_same_existing_ctrl, initial_run=initial_run, 
             calc_gas_from_power=calc_gas_from_power, **kwargs)
 
-        self.net = multinet
         self.name = name
         self.__load_regulation_multiplier = 1
         self.__heat_regulation_multiplier = initial_heat_regulation
@@ -138,6 +137,20 @@ class CHPControlMultiEnergy(G2PControlMultiEnergy):
         self.storage = None
         self.heat_energy = 0
         self.power_gen = 0
+        
+        heat_net = multinet['nets'][self.__name_heat_net]
+        power_net = multinet['nets'][self.name_net_power]
+        gas_net = multinet['nets'][self.name_net_gas]
+
+        from_heat_junc = heat_net.heat_exchanger.loc[self.__heat_exchanger_id, 'from_junction']
+        to_heat_junc = heat_net.heat_exchanger.loc[self.__heat_exchanger_id, 'to_junction']
+        bus_gen = power_net[self.elm_type_power].loc[self.elm_idx_power, 'bus']
+        junction_sink = gas_net.sink.loc[self.elm_idx_gas, 'junction']
+
+        self._pipes_heat = dict(find_pipes_for_junction(heat_net, from_heat_junc, 'from_junction'))
+        self._pipes_heat.update(find_pipes_for_junction(heat_net, to_heat_junc, 'to_junction'))
+        self._line_power = find_lines_for_bus(power_net, bus_gen)
+        self._pipes_gas = find_pipes_for_junction(gas_net, junction_sink)
     
     def get_gas_consume(self, multinet):
         try:
@@ -226,23 +239,10 @@ class CHPControlMultiEnergy(G2PControlMultiEnergy):
         :return: dict, energy-network-name -> edges
         :rtype: Dict
         """
-        heat_net = self.net['nets'][self.__name_heat_net]
-        power_net = self.net['nets'][self.name_net_power]
-        gas_net = self.net['nets'][self.name_net_gas]
-
-        from_heat_junc = heat_net.heat_exchanger.loc[self.__heat_exchanger_id, 'from_junction']
-        to_heat_junc = heat_net.heat_exchanger.loc[self.__heat_exchanger_id, 'to_junction']
-        bus_gen = power_net[self.elm_type_power].loc[self.elm_idx_power, 'bus']
-        junction_sink = gas_net.sink.loc[self.elm_idx_gas, 'junction']
-
-        pipes_heat = dict(find_pipes_for_junction(heat_net, from_heat_junc, 'from_junction'))
-        pipes_heat.update(find_pipes_for_junction(heat_net, to_heat_junc, 'to_junction'))
-        line_power = find_lines_for_bus(power_net, bus_gen)
-        pipes_gas = find_pipes_for_junction(gas_net, junction_sink)
         
-        return {self.name_net_power: line_power, 
-                self.name_net_gas: pipes_gas, 
-                self.__name_heat_net: pipes_heat}
+        return {self.name_net_power: self._line_power, 
+                self.name_net_gas: self._pipes_gas, 
+                self.__name_heat_net: self._pipes_heat}
 
     @property
     def nodes(self):
@@ -285,12 +285,21 @@ class RegulatedG2PControlMultiEnergy(G2PControlMultiEnergy):
                          name_gas_net=name_gas_net, in_service=in_service, order=order, level=level, 
                          drop_same_existing_ctrl=drop_same_existing_ctrl, initial_run=initial_run, **kwargs)
 
-        self.net = multinet
         self.__load_regulation_multiplier = 1
         self.name = name
         self.power_gen = 0
         self._desync = desync_mode
 
+        
+        power_net = multinet['nets'][self.name_net_power]
+        gas_net = multinet['nets'][self.name_net_gas]
+
+        bus_gen = power_net[self.elm_type_power].loc[self.elm_idx_power, 'bus']
+        junction_sink = gas_net.sink.loc[self.elm_idx_gas, 'junction']
+
+        self._line_power = find_lines_for_bus(power_net, bus_gen) 
+        self._pipes_gas = find_pipes_for_junction(gas_net, junction_sink)
+
     def get_gas_consume(self, multinet):
         try:
             gas_sink = multinet['nets'][self.name_net_gas].sink.at[self.elm_idx_gas,
@@ -350,17 +359,7 @@ class RegulatedG2PControlMultiEnergy(G2PControlMultiEnergy):
 
     @property
     def edges(self):
-        power_net = self.net['nets'][self.name_net_power]
-        gas_net = self.net['nets'][self.name_net_gas]
-
-        bus_gen = power_net[self.elm_type_power].loc[self.elm_idx_power, 'bus']
-
-        junction_sink = gas_net.sink.loc[self.elm_idx_gas, 'junction']
-
-        line_power = find_lines_for_bus(power_net, bus_gen) 
-        pipes_gas = find_pipes_for_junction(gas_net, junction_sink)
-        
-        return {self.name_net_power: line_power, self.name_net_gas: pipes_gas}
+        return {self.name_net_power: self._line_power, self.name_net_gas: self._pipes_gas}
 
     @property
     def regulation(self):
@@ -381,16 +380,23 @@ class RegulatedP2GControlMultiEnergy(P2GControlMultiEnergy):
                          name_gas_net=name_gas_net, in_service=in_service, order=order, level=level, 
                          drop_same_existing_ctrl=drop_same_existing_ctrl, initial_run=initial_run, **kwargs)
 
-        self.net = multinet
         self.__load_regulation_multiplier = 1
         self.name = name
         self._desync = desync_mode
+        power_net = multinet['nets'][self.name_net_power]
+        gas_net = multinet['nets'][self.name_net_gas]
+
+        bus_gen = power_net.load.loc[self.elm_idx_power, 'bus']
+        junction_sink = gas_net.source.loc[self.elm_idx_gas, 'junction']
+
+        self._line_power = find_lines_for_bus(power_net, bus_gen) 
+        self._pipes_gas = find_pipes_for_junction(gas_net, junction_sink)
 
     def get_power_load(self, multinet):
         try:
             power_load = multinet[NETS_ACCESS][self.name_net_power].load.at[self.elm_idx_power, P_MW]
         except (ValueError, TypeError):
-            power_load = multinet[NETS_ACCESS][self.name_net_power].load.loc[self.elm_idx_power, P_MW].values
+            power_load = multinet[NETS_ACCESS][self.name_net_power].load.loc[self.elm_idx_power, P_MW].values[:]
         return power_load
 
     def get_source_production(self, multinet):
@@ -404,7 +410,7 @@ class RegulatedP2GControlMultiEnergy(P2GControlMultiEnergy):
         if time == 0:
             self.ref_power_load = self.get_power_load(multinet)
         
-        self.mdot_kg_per_s = self.__load_regulation_multiplier * self.get_power_load(multinet) * self.conversion_factor_mw_to_kgps() * self.efficiency
+        self.mdot_kg_per_s = self.__load_regulation_multiplier * self.get_power_load(multinet) * self.conversion_factor_mw_to_kgps()[0] * self.efficiency
         self.power_load = self.ref_power_load * self.__load_regulation_multiplier
         self.write_to_net(multinet)
 
@@ -420,10 +426,8 @@ class RegulatedP2GControlMultiEnergy(P2GControlMultiEnergy):
         except (ValueError, TypeError):
             multinet[NETS_ACCESS][self.name_net_power].load.loc[self.elm_idx_power, P_MW].values[:] = self.power_load
 
-
     def control_step(self, _):
-        self.applied = True
-    
+        self.applied = True    
     
     @property
     def nodes(self):
@@ -432,16 +436,8 @@ class RegulatedP2GControlMultiEnergy(P2GControlMultiEnergy):
 
     @property
     def edges(self):
-        power_net = self.net['nets'][self.name_net_power]
-        gas_net = self.net['nets'][self.name_net_gas]
-
-        bus_gen = power_net.load.loc[self.elm_idx_power, 'bus']
-        junction_sink = gas_net.source.loc[self.elm_idx_gas, 'junction']
-
-        line_power = find_lines_for_bus(power_net, bus_gen) 
-        pipes_gas = find_pipes_for_junction(gas_net, junction_sink)
         
-        return {self.name_net_power: line_power, self.name_net_gas: pipes_gas}
+        return {self.name_net_power: self._line_power, self.name_net_gas: self._pipes_gas}
 
     @property
     def regulation(self):
